import base64
import json
import os

from flask import request, jsonify, current_app, g
from flask_restx import Resource, Namespace, reqparse
from sqlalchemy import select, insert, update, delete, func, and_
from sqlalchemy.orm import Session
from werkzeug.datastructures import FileStorage

from app.defines import Module, OperationType, StatesCode
from app.modle.information import FloorConfiguration, StaffConfiguration, SecurityPerson, FloorConfigurationHistory, \
    StaffConfigurationStaff
from app.utils.jwt_util import login_required
from app.utils.save_log import save_log
from app.utils.util import to_dict, cn_now
from config import Config

ns = Namespace('information', description='信息管理接口')

config = Config()


def information_list(rows):
    res = []
    for row in rows:
        res.append({"id": row.id, "name": row.name})

    return res


def upload_history(floor_configuration_id, url):
    """
    记录楼层表地图信息配置历史记录
    :param floor_configuration_id: 楼层表地图信息配置id
    :param url: 图片url
    :return:
    """
    with Session(current_app.engine) as session:
        stmt = select(FloorConfigurationHistory.url).where(
            FloorConfigurationHistory.floor_configuration == floor_configuration_id)
        res = session.execute(stmt).scalars().first()
        if res:
            res = json.loads(res)
            res.append({'time': cn_now(), 'url': url, 'account': g.user_name})
            up_stmt = update(FloorConfigurationHistory).where(
                FloorConfigurationHistory.floor_configuration == floor_configuration_id).values(url=json.dumps(res))

        else:
            res = []
            res.append({'time': cn_now(), 'url': url, 'account': g.user_name})
            up_stmt = insert(FloorConfigurationHistory).values(
                floor_configuration=floor_configuration_id,
                url=json.dumps(res)
            )
        session.execute(up_stmt)
        session.commit()


@ns.route('/information_list')
class InformationTypeApi(Resource):
    # method_decorators = [login_required]

    @ns.doc(id='information_list', description='信息列表')
    def get(self):
        """获取信息列表"""
        information_type = config.common.INFORMATION_TYPE

        data = {'楼层地图信息配置': [], '安保人员信息配置': []}
        try:
            with Session(current_app.engine) as session:

                for information in information_type:
                    if information == '楼层地图信息配置':

                        stmt = select(FloorConfiguration.id, FloorConfiguration.name)

                        data[information] = information_list(session.execute(stmt))

                    elif information == '安保人员信息配置':

                        stmt = select(StaffConfiguration.id, StaffConfiguration.name)

                        data[information] = information_list(session.execute(stmt))

            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功', data=data)

        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


floor_configuration_details = reqparse.RequestParser(bundle_errors=True)
floor_configuration_details.add_argument(name='id', type=str, location='args', required=False, help='配置id')

floor_configuration = reqparse.RequestParser(bundle_errors=True)
floor_configuration.add_argument(name='id', type=str, location='form', required=False, help='配置id')
floor_configuration.add_argument(name='name', type=str, location='form', required=False, help='配置名称')
floor_configuration.add_argument(name='location', type=str, location='form', required=False, help='楼层位置')
floor_configuration.add_argument(name='use', type=str, location='form', required=False, help='楼层用途')
floor_configuration.add_argument(name='remark', type=str, location='form', required=False, help='备注')
floor_configuration.add_argument(name='attachment_url', type=str, location='form', required=False, help='附件url')


@ns.route('/floor_configuration')
class FloorConfigurationApi(Resource):
    # method_decorators = [login_required]

    @ns.doc(id='floor_configuration', description='获取楼层地图信息配置')
    @ns.expect(floor_configuration_details)
    def get(self):
        """获取楼层地图信息配置"""

        floor_configuration_id = request.args.get('id')

        try:
            with Session(current_app.engine) as session:
                stmt = select(FloorConfiguration).where(FloorConfiguration.id == floor_configuration_id)
                results = session.execute(stmt).scalars().all()

            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功', data=to_dict(results))
        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='add_floor_configuration', description='添加楼层地图信息配置')
    @ns.expect(floor_configuration)
    def post(self):
        """添加楼层地图信息配置"""
        name = request.form.get('name')
        location = request.form.get('location')
        use = request.form.get('use')
        remark = request.form.get('remark')
        attachment_url = request.form.get('attachment_url')

        if name is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="配置名不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = insert(FloorConfiguration).values(
                    name=name,
                    location=location,
                    use=use,
                    remark=remark,
                    attachment=attachment_url,
                    account=g.user_name
                )
                result = session.execute(stmt)
                session.commit()
                # 添加历史记录
                upload_history(result.inserted_primary_key[0], attachment_url)

            save_log(request, Module.INFORMATION, OperationType.ADD, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='添加成功')
        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.ADD, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='update_floor_configuration', description='修改楼层地图信息配置')
    @ns.expect(floor_configuration)
    def put(self):
        """修改楼层地图信息配置"""

        floor_configuration_id = request.form.get('id')
        name = request.form.get('name')
        location = request.form.get('location')
        use = request.form.get('use')
        remark = request.form.get('remark')
        attachment_url = request.form.get('attachment_url')

        if name is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="配置名不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = update(FloorConfiguration).where(FloorConfiguration.id == floor_configuration_id).values(
                    name=name,
                    location=location,
                    use=use,
                    remark=remark,
                    attachment=attachment_url,
                    account=g.user_name
                )
                session.execute(stmt)
                session.commit()
                # 记录历史
                upload_history(floor_configuration_id, attachment_url)

            save_log(request, Module.INFORMATION, OperationType.UPDATE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='修改成功')
        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.UPDATE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='delete_floor_configuration', description='删除楼层地图信息配置')
    @ns.expect(floor_configuration)
    def delete(self):
        """删除楼层地图信息配置"""
        floor_configuration_id = request.form.get('id')

        if floor_configuration_id is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="配置id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = delete(FloorConfiguration).where(FloorConfiguration.id == floor_configuration_id)
                session.execute(stmt)
                session.commit()

            save_log(request, Module.INFORMATION, OperationType.DELETE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message="删除成功")
        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.DELETE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


security_person = reqparse.RequestParser(bundle_errors=True)
security_person.add_argument(name='type_id', type=str, location='args', required=False,
                             help='人员类别id 1 外协员工')


@ns.route('/security_person')
class GetSecurityPersonApi(Resource):
    # method_decorators = [login_required]
    @ns.expect(security_person)
    def get(self):
        """获取人员名单"""
        type_id = request.args.get('type_id')

        if type_id:
            type_id = int(type_id)
        else:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="类别id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = select(SecurityPerson).where(SecurityPerson.type == type_id)
                results = session.execute(stmt).scalars().all()

            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功', data=to_dict(results))
        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


staff_configuration_details = reqparse.RequestParser(bundle_errors=True)
staff_configuration_details.add_argument(name='id', type=str, location='args', required=False, help='配置id')
staff_configuration_details.add_argument(name='page_size', type=int, location='args', required=False, help='每页记录数量，默认:20')
staff_configuration_details.add_argument(name='page', type=int, location='args', required=False, help='第几页')

configuration = reqparse.RequestParser(bundle_errors=True)
configuration.add_argument(name='id', type=str, location='form', required=False, help='配置id')
configuration.add_argument(name='name', type=str, location='form', required=False, help='配置名称')


@ns.route('/staff_configuration')
class StaffConfigurationApi(Resource):
    # method_decorators = [login_required]

    @ns.doc(id='staff_configuration', description='获取人员配置信息')
    @ns.expect(staff_configuration_details)
    def get(self):
        """获取安保人员信息配置"""

        staff_configuration_id = request.args.get('id')
        page_size = int(request.args.get('page_size', 20))
        page = int(request.args.get('page', 1))

        if staff_configuration_id is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="配置id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = select(StaffConfigurationStaff.staff,
                              func.max(StaffConfigurationStaff.update_time)) \
                    .where(StaffConfigurationStaff.Configuration_id == staff_configuration_id) \
                    .group_by(StaffConfigurationStaff.staff)
                staff_res = session.execute(stmt).all()

                data = []
                count = 0
                for staff in staff_res:
                    count_stmt = select(func.count(StaffConfigurationStaff.id)).where(and_(StaffConfigurationStaff.staff == staff[0],
                                                                      StaffConfigurationStaff.update_time == staff[1]))
                    count_res = session.execute(count_stmt).scalars().first()
                    count += count_res
                    stmt = select(StaffConfigurationStaff).where(and_(StaffConfigurationStaff.staff == staff[0],
                                                                      StaffConfigurationStaff.update_time == staff[1]))\
                                                            .offset(page_size * (page - 1)).limit(page_size)
                    data.append(session.execute(stmt).scalars().first())

            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功',total=count, data=to_dict(data))
        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='add_staff_configuration', description='添加人员配置信息')
    @ns.expect(configuration)
    def post(self):
        """添加安保人员信息配置"""
        name = request.form.get('name')

        try:
            with Session(current_app.engine) as session:
                stmt = insert(StaffConfiguration).values(
                    name=name,
                )
                session.execute(stmt)
                session.commit()

            # # 添加历史记录
            # staff_upload_history(result.inserted_primary_key[0], staff_list, duty_time)

            save_log(request, Module.INFORMATION, OperationType.ADD, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功')

        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.ADD, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='delete_staff_configuration', description='删除人员配置信息')
    @ns.expect(floor_configuration)
    def delete(self):
        """删除安保人员信息配置"""

        staff_configuration_id = request.form.get('id')

        if staff_configuration_id is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="配置id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = delete(StaffConfiguration).where(StaffConfiguration.id == staff_configuration_id)
                session.execute(stmt)
                session.commit()

            save_log(request, Module.INFORMATION, OperationType.DELETE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功')

        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.DELETE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


del_staff = reqparse.RequestParser(bundle_errors=True)
del_staff.add_argument(name='id', type=str, location='form', required=False, help='id')

staff_configuration = reqparse.RequestParser(bundle_errors=True)
staff_configuration.add_argument(name='id', type=str, location='form', required=False, help='配置id')
staff_configuration.add_argument(name='responsibility_range', type=str, location='form', required=False,
                                 help='职责范围')
staff_configuration.add_argument(name='staff_list', type=str, location='form', required=False, help='人员名单')
staff_configuration.add_argument(name='remark', type=str, location='form', required=False, help='备注')
staff_configuration.add_argument(name='duty_time', type=str, location='form', required=False,
                                 help="值班日期：{'start_time': '', end_time: ''}")


@ns.route('/staff')
class DelStaff(Resource):
    @ns.doc(id='staff_configuration', description='新建人员配置')
    @ns.expect(staff_configuration)
    def post(self):
        """新建人员配置"""
        configuration_id = request.form.get('id')
        responsibility_range = request.form.get('responsibility_range')
        staff_list = request.form.get('staff_list')
        remark = request.form.get('remark')
        duty_time = request.form.get('duty_time')

        if staff_list:
            staff_list = json.loads(staff_list)
        if duty_time:
            duty_time = json.loads(duty_time)

        data = []
        for staff in staff_list:
            data.append(
                {
                    'Configuration_id': configuration_id,
                    'responsibility_range': responsibility_range,
                    'staff': staff.get('id'),
                    'remark': remark,
                    'duty_time': duty_time,
                    'staff_details': json.dumps(staff.get('details')),
                    'update_time': cn_now()
                }
            )
        with Session(current_app.engine) as session:
            session.execute(
                insert(StaffConfigurationStaff), data
            )
            session.commit()
        save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.SUCCESS)

        return jsonify(code=StatesCode.SUCCESS, message='成功')

    @ns.doc(description='删除人员')
    @ns.expect(del_staff)
    def delete(self):
        """删除人员"""
        staff_configuration_id = request.form.get('id')

        if staff_configuration_id is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = delete(StaffConfigurationStaff).where(StaffConfigurationStaff.id == staff_configuration_id)
                session.execute(stmt)
                session.commit()

            save_log(request, Module.INFORMATION, OperationType.UPDATE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功')

        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.DELETE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


upload_pictures = reqparse.RequestParser(bundle_errors=True)
upload_pictures.add_argument(name='file', type=FileStorage, location='files', required=False, help='图片')


@ns.route('/upload_pictures')
class UploadPictures(Resource):
    @ns.doc(description='图片上传')
    @ns.expect(upload_pictures)
    def post(self):
        """上传图片"""
        file = request.files.get('file')
        if file:
            file.save(os.path.join(config.common.SAVE_FILE_PATH, file.filename))
            file_url = config.common.FILE_URL + file.filename
        else:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message='图片不能为空')

        return jsonify(code=StatesCode.SUCCESS, message='成功', data=file_url)


get_floor_configuration_history = reqparse.RequestParser(bundle_errors=True)
get_floor_configuration_history.add_argument(name='floor_configuration_id', type=str, location='args', required=False,
                                             help='楼层地图信息配置id')


@ns.route('/get_floor_configuration_history')
class GetFloorConfigurationHistory(Resource):

    @ns.doc(description='获取楼层地图信息配置历史记录')
    @ns.expect(get_floor_configuration_history)
    def get(self):
        """获取楼层地图信息配置历史记录"""

        floor_configuration_id = request.args.get('floor_configuration_id')

        if floor_configuration_id is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="配置id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = select(FloorConfigurationHistory.url).where(
                    FloorConfigurationHistory.floor_configuration == floor_configuration_id)
                results = session.execute(stmt).scalars().first()

            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功', data=results)

        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


@ns.route('/staff_configuration_history')
class GetStaffConfigurationHistory(Resource):

    @ns.doc(description='获取安保人员信息配置历史记录')
    @ns.expect(staff_configuration_details)
    def get(self):
        """获取人员历史记录"""
        staff_configuration_id = request.args.get('id')

        if staff_configuration_id is None:
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message="配置id不能为空")

        try:
            results = []
            with Session(current_app.engine) as session:
                stmt = select(StaffConfigurationStaff.update_time) \
                    .where(StaffConfigurationStaff.Configuration_id == staff_configuration_id) \
                    .group_by(StaffConfigurationStaff.update_time)
                update_time_res = session.execute(stmt).fetchall()

                for update_time in update_time_res:
                    stmt = select(StaffConfigurationStaff).where(StaffConfigurationStaff.update_time == update_time[0])
                    results.append({'date': update_time[0], 'data': to_dict(session.execute(stmt).scalars().all())})

            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功', data=results)

        except Exception as e:
            save_log(request, Module.INFORMATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))
